(* Copyright (C) 2005  Eric C. Cooper <ecc@cmu.edu>
   Released under the GNU General Public License *)

open GdkEvent
module K = GdkKeysyms

let handle_key event =
  let key = Key.keyval event in
  if key = K._Q || key = K._q || key = K._Escape then GMain.quit ();
  true

class app =
  let window = GWindow.window ~width: Screen.width ~height: Screen.height () in
  let area = GlGtk.area [`DOUBLEBUFFER; `RGBA] ~packing: window#add () in
  object (self)
    val mutable aspect = 1.0

    method init () =
      Screen.init ();
      GlClear.color (0.0, 0.0, 0.0);
      GlDraw.color (1.0, 1.0, 1.0);
      self#reshape ~width: Screen.width ~height: Screen.height

    method reshape ~width ~height =
      aspect <- float width /. float height;
      GlDraw.viewport ~x: 0 ~y: 0 ~w: width ~h: height;
      GlMat.mode `projection;
      GlMat.load_identity ();
      GluMat.perspective ~fovy: 45.0 ~aspect ~z: (2.0, 85.0);
      GlMat.mode `modelview

    method transform () = ()

    method display () =
      GlClear.clear [`color; `depth];
      GlMat.load_identity ();
      let eye = 0.0, 0.0, -12.0 in
      let center = 0.0, 0.0, 0.0 in
      let up = 0.0, 1.0, 0.0 in
      GluMat.look_at ~eye ~center ~up;
      GlMat.push ();
      self#transform ();
      Screen.display ~w: (10.0 *. aspect) ~h: 10.0;
      GlMat.pop ();
      area#swap_buffers ()

    method visible () = area#misc#visible

    method interval = 0

    method idle () = ()

    method main () =
      window#connect#destroy ~callback: (fun () -> exit 0);
      window#event#add [`KEY_PRESS];
      window#event#connect#key_press ~callback: handle_key;
      window#fullscreen ();
      area#connect#realize ~callback: self#init;
      area#connect#display ~callback: self#display;
      area#connect#reshape ~callback: self#reshape;
      window#show ();
      let ms = self#interval in
      let callback () = self#idle (); true in
      if ms <> 0 then ignore (GMain.Timeout.add ~ms ~callback);
      GMain.main ()
  end
