(* Copyright (C) 2005  Eric C. Cooper <ecc@cmu.edu>
   Released under the GNU General Public License *)

let hdaps_file = "/sys/devices/platform/hdaps/position"
let threshold = 4

let hdaps_position () =
  try
    let chan = open_in hdaps_file in
    let coords = Scanf.fscanf chan "(%d,%d)\n" (fun x y -> x, y) in
    close_in chan;
    coords
  with Sys_error msg ->
    prerr_endline msg;
    exit 1

(* Tweak these for better accuracy *)

let x_coeff = 0.3
let y_coeff = 0.2

let pi = acos 1.0 *. 2.0

let x0, y0 = hdaps_position ()

let app = object (self)
  inherit Gl_window.app

  val mutable px = x0
  val mutable py = y0

  method transform () =
    let angle = x_coeff *. float px in
    GlMat.rotate ~angle ~z: (-1.0) ();
    let angle = y_coeff *. float py in
    GlMat.translate ~z: (cos (angle *. pi /. 180.0)) ();
    GlMat.rotate ~angle ~x: 1.0 ()

  method interval = 20

  method idle () =
    let x, y = hdaps_position () in
    let x = x - x0 and y = y - y0 in
    let dx = abs (x - px) and dy = abs (y - py) in
    if dx > threshold || dy > threshold then
      begin
	px <- x;
	py <- y;
	if self#visible () then self#display ()
      end
end

let () = app#main ()
