(* Copyright (C) 2005  Eric C. Cooper <ecc@cmu.edu>
   Released under the GNU General Public License *)

let _ = GMain.init ()

let width = Gdk.Screen.width ()
let height = Gdk.Screen.height ()

let screenshot = GdkPixbuf.create ~width ~height ()
let () =
  GdkPixbuf.get_from_drawable (Gdk.Window.root_parent ()) ~dest: screenshot

let texture_size = 64  (* small but guaranteed to work *)

let extra_x = width mod texture_size
let x_steps = width / texture_size + (if extra_x = 0 then 0 else 1)

let extra_y = height mod texture_size
let y_steps = height / texture_size + (if extra_y = 0 then 0 else 1)

let make_tile i j =
  let tile = GdkPixbuf.create ~width: texture_size ~height: texture_size () in
  GdkPixbuf.fill tile Int32.zero;
  let src_x = i * texture_size in
  let src_y = j * texture_size in
  let width =
    if extra_x <> 0 && i = x_steps - 1 then extra_x else texture_size
  in
  let height =
    if extra_y <> 0 && j = y_steps - 1 then extra_y else texture_size
  in
  GdkPixbuf.copy_area ~dest: tile ~width ~height ~src_x ~src_y screenshot;
  let str = Gpointer.string_of_region (GdkPixbuf.get_pixels tile) in
  GlPix.of_raw (Raw.of_string str ~kind: `ubyte) ~format: `rgb
    ~width: texture_size ~height: texture_size

let tile_to_texture tile =
  GluMisc.build_2d_mipmaps tile;
  List.iter (GlTex.parameter ~target: `texture_2d)
    [`mag_filter `linear; `min_filter `linear_mipmap_linear;
     `wrap_s `repeat; `wrap_t `repeat]

let make_textures () =
  let make i j =
    let id = GlTex.gen_texture () in
    GlTex.bind_texture ~target: `texture_2d id;
    tile_to_texture (make_tile i j);
    id
  in
  GlPix.store (`unpack_alignment 1);
  Array.init x_steps (fun i -> (Array.init y_steps (make i)))

let textures = ref [||]

let init () =
  Gl.enable `texture_2d;
  textures := make_textures ()

let display ~w ~h =
  let x0 = w /. 2.0 and y0 = h /. 2.0 in
  let tw = float texture_size *. w /. float width in
  let th = float texture_size *. h /. float height in
  for i = 1 to x_steps do
    for j = 1 to y_steps do
      GlTex.bind_texture ~target: `texture_2d !textures.(i-1).(j-1);
      let x = x0 -. float i *. tw in
      let x' = x +. tw in
      let y = y0 -. float j *. th in
      let y' = y +. th in
      GlDraw.begins `quads;
      GlTex.coord2 (0.0, 0.0); GlDraw.vertex2 (x', y');
      GlTex.coord2 (0.0, 1.0); GlDraw.vertex2 (x', y);
      GlTex.coord2 (1.0, 1.0); GlDraw.vertex2 (x, y);
      GlTex.coord2 (1.0, 0.0); GlDraw.vertex2 (x, y');
      GlDraw.ends ()
    done
  done
